==> haystack-0.5.2.tar.gz: lib/src/HDate.dart <==
///
/// Copyright (c) 2018, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hdate;

import 'package:haystack/src/str-utils.dart';
import 'package:quiver/core.dart';

import 'HDateTime.dart';
import 'HTime.dart';
import 'HTimeZone.dart';
import 'HVal.dart';
import 'ParseError.dart';

/// HDate models a date (day in year) tag value.
///
/// See [Project Haystack](http://project-haystack.org/doc/TagModel#tagKinds)
class HDate extends HVal {

  /// Construct from basic fields 
  factory HDate(int year, int month, int day) {
    if (year < 1900) throw new FormatException("Invalid year", year);
    if (month < 1 || month > 12) throw new FormatException("Invalid month", month);
    if (day < 1 || day > 31) throw new FormatException("Invalid day", day);
    return new HDate._(year, month, day);
  }

  /// Construct from Dart DateTime instance 
  factory HDate.fromDateTime(DateTime dt) => new HDate(dt.year, dt.month, dt.day);

  /// Parse from string fomat "YYYY-MM-DD" or raise ParseError 
  factory HDate.fromString(String s) {
    ParseError err = new ParseError(s);
    if (s.length != "YYYY-MM-DD".length) throw err;
    int year, month, day;

    try { year = int.parse(s.substring(0, 4)); }
    catch (e) { throw new ParseError("Invalid year: $s"); }
    if (s.codeUnitAt(4) != MINUS_CHAR) throw err;

    try { month = int.parse(s.substring(5, 7)); }
    catch (e) { throw new ParseError("Invalid month: $s"); }
    if (s.codeUnitAt(7) != MINUS_CHAR) throw err;
    try { day = int.parse(s.substring(8)); }
    catch (e) { throw new ParseError("Invalid day: $s"); }
    return new HDate(year, month, day);
  }

  /// Get HDate for current time in default timezone 
  static HDate today([HTimeZone tz]) {
    return new HDateTime.now(tz).date;
  }

  HDate._(this.year, this.month, this.day)
      : hashCode = hash3(year, month, day);

  /// Four digit year such as 2011 
  final int year;

  /// Month as 1-12 (Jan is 1, Dec is 12) 
  final int month;

  /// Day of month as 1-31 
  final int day;

  /// Hash is based on year, month, day 
  @override
  final int hashCode;

  /// Equals is based on year, month, day 
  @override
  bool operator ==(Object that) {
    if (that is! HDate) return false;
    HDate x = that as HDate;
    return year == x.year && month == x.month && day == x.day;
  }

  /// Return sort order as negative, 0, or positive 
  @override
  int compareTo(Object that) {
    HDate x = that as HDate;
    if (year < x.year)   return -1; else if (year > x.year)   return 1;
    if (month < x.month) return -1; else if (month > x.month) return 1;
    if (day < x.day)     return -1; else if (day > x.day)     return 1;
    return 0;
  }

  /// Encode as "d:YYYY-MM-DD" 
  @override
  String toJson() {
    StringBuffer s = new StringBuffer();
    s.write("d:");
    encode(s);
    return s.toString();
  }

  /// Encode as "YYYY-MM-DD" 
  @override
  String toZinc() {
    StringBuffer s = new StringBuffer();
    encode(s);
    return s.toString();
  }

  /// Package private implementation shared with HDateTime 
  void encode(StringBuffer s) {
    s.write(year);
    s.write('-');

    if (month < 10) s.write('0');
    s.write(month); s.write('-');

    if (day < 10) s.write('0');
    s.write(day);
  }

  /// Convert this date into HDateTime for midnight in given timezone. 
  HDateTime midnight(HTimeZone tz) {
    return new HDateTime(this, HTime.MIDNIGHT, tz);
  }

  /// Return date in future given number of days 
  HDate plusDays(int numDays) {
    if (numDays == 0) return this;
    if (numDays < 0) return minusDays(-numDays);
    int year  = this.year;
    int month = this.month;
    int day   = this.day;

    for (; numDays > 0; --numDays) {
      day++;
      if (day > daysInMonth(year, month)) {
        day = 1;
        month++;
        if (month > 12) { month = 1; year++; }
      }
    }
    return new HDate(year, month, day);
  }

  /// Return date in past given number of days 
  HDate minusDays(int numDays) {
    if (numDays == 0) return this;
    if (numDays < 0) return plusDays(-numDays);
    int year  = this.year;
    int month = this.month;
    int day   = this.day;

    for (; numDays > 0; --numDays) {
      day--;
      if (day <= 0) {
        month--;
        if (month < 1) { month = 12; year--; }
        day = daysInMonth(year, month);
      }
    }
    return new HDate(year, month, day);
  }

  /// Return if given year a leap year 
  static bool isLeapYear(int year) => ((year & 3) == 0) && ((year % 100 != 0) || (year % 400 == 0));

  /// Return number of days in given year (xxxx) and month (1-12) 
  static int daysInMonth(int year, int mon) => isLeapYear(year) ? daysInMonLeap[mon] : daysInMon[mon];

  /// Return day of week: Sunday is 1, Saturday is 7 
  int weekday() => new DateTime(year, month, day).weekday;

  static final List<int> daysInMon     = new List<int>.from(<int>[ -1, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ]);
  static final List<int> daysInMonLeap = new List<int>.from(<int>[ -1, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ]);
}

==> haystack-0.6.1.tar.gz: lib/src/HDate.dart <==
///
/// Copyright (c) 2018, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hdate;

import 'package:haystack/src/string_utils.dart';
import 'package:quiver/core.dart';

import 'HDateTime.dart';
import 'HTime.dart';
import 'HTimeZone.dart';
import 'HVal.dart';
import 'ParseError.dart';

/// HDate models a date (day in year) tag value.
///
/// See [Project Haystack](http://project-haystack.org/doc/TagModel#tagKinds)
class HDate extends HVal {
  /// Construct from basic fields
  factory HDate(int year, int month, int day) {
    if (year < 1900) throw new FormatException("Invalid year", year);
    if (month < 1 || month > 12)
      throw new FormatException("Invalid month", month);
    if (day < 1 || day > 31) throw new FormatException("Invalid day", day);
    return new HDate._(year, month, day);
  }

  /// Construct from Dart DateTime instance
  factory HDate.fromDateTime(DateTime dt) =>
      new HDate(dt.year, dt.month, dt.day);

  /// Parse from string fomat "YYYY-MM-DD" or raise ParseError
  factory HDate.fromString(String s) {
    ParseError err = new ParseError(s);
    if (s.length != "YYYY-MM-DD".length) throw err;
    int year, month, day;

    try {
      year = int.parse(s.substring(0, 4));
    } catch (e) {
      throw new ParseError("Invalid year: $s");
    }
    if (s.codeUnitAt(4) != MINUS_CHAR) throw err;

    try {
      month = int.parse(s.substring(5, 7));
    } catch (e) {
      throw new ParseError("Invalid month: $s");
    }
    if (s.codeUnitAt(7) != MINUS_CHAR) throw err;
    try {
      day = int.parse(s.substring(8));
    } catch (e) {
      throw new ParseError("Invalid day: $s");
    }
    return new HDate(year, month, day);
  }

  /// Get HDate for current time in default timezone
  static HDate today([HTimeZone tz]) {
    return new HDateTime.now(tz).date;
  }

  HDate._(this.year, this.month, this.day);

  /// Four digit year such as 2011
  final int year;

  /// Month as 1-12 (Jan is 1, Dec is 12)
  final int month;

  /// Day of month as 1-31
  final int day;

  /// Hash is based on year, month, day
  int _hashCode;
  @override
  int get hashCode {
    if (_hashCode == null) {
      _hashCode = hash3(year, month, day);
    }
    return _hashCode;
  }

  /// Equals is based on year, month, day
  @override
  bool operator ==(Object that) {
    if (that is! HDate) return false;
    HDate x = that as HDate;
    return year == x.year && month == x.month && day == x.day;
  }

  /// Return sort order as negative, 0, or positive
  @override
  int compareTo(Object that) {
    HDate x = that as HDate;
    if (year < x.year)
      return -1;
    else if (year > x.year) return 1;
    if (month < x.month)
      return -1;
    else if (month > x.month) return 1;
    if (day < x.day)
      return -1;
    else if (day > x.day) return 1;
    return 0;
  }

  /// Encode as "d:YYYY-MM-DD"
  @override
  String toJson() {
    StringBuffer s = new StringBuffer();
    s.write("d:");
    encode(s);
    return s.toString();
  }

  /// Encode as "YYYY-MM-DD"
  @override
  String toZinc() {
    StringBuffer s = new StringBuffer();
    encode(s);
    return s.toString();
  }

  /// Package private implementation shared with HDateTime
  void encode(StringBuffer s) {
    s.write(year);
    s.write('-');

    if (month < 10) s.write('0');
    s.write(month);
    s.write('-');

    if (day < 10) s.write('0');
    s.write(day);
  }

  /// Convert this date into HDateTime for midnight in given timezone.
  HDateTime midnight(HTimeZone tz) {
    return new HDateTime(this, HTime.MIDNIGHT, tz);
  }

  /// Return date in future given number of days
  HDate plusDays(int numDays) {
    if (numDays == 0) return this;
    if (numDays < 0) return minusDays(-numDays);
    int year = this.year;
    int month = this.month;
    int day = this.day;

    for (; numDays > 0; --numDays) {
      day++;
      if (day > daysInMonth(year, month)) {
        day = 1;
        month++;
        if (month > 12) {
          month = 1;
          year++;
        }
      }
    }
    return new HDate(year, month, day);
  }

  /// Return date in past given number of days
  HDate minusDays(int numDays) {
    if (numDays == 0) return this;
    if (numDays < 0) return plusDays(-numDays);
    int year = this.year;
    int month = this.month;
    int day = this.day;

    for (; numDays > 0; --numDays) {
      day--;
      if (day <= 0) {
        month--;
        if (month < 1) {
          month = 12;
          year--;
        }
        day = daysInMonth(year, month);
      }
    }
    return new HDate(year, month, day);
  }

  /// Return if given year a leap year
  static bool isLeapYear(int year) =>
      ((year & 3) == 0) && ((year % 100 != 0) || (year % 400 == 0));

  /// Return number of days in given year (xxxx) and month (1-12)
  static int daysInMonth(int year, int mon) =>
      isLeapYear(year) ? daysInMonLeap[mon] : daysInMon[mon];

  /// Return day of week: Sunday is 1, Saturday is 7
  int weekday() => new DateTime(year, month, day).weekday;

  static final List<int> daysInMon = new List<int>.from(
      <int>[-1, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]);
  static final List<int> daysInMonLeap = new List<int>.from(
      <int>[-1, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]);
}
